# HealMates release 7 by David Bowland
# ./addons/eventscripts/healmates/healmates.py

# >>> To configure this addon please see healmates.cfg (created when the addon is first loaded) <<<

"""
Allows players to heal surrounding teammates. Compatible with bots! Requires only ES 2.0.0.248+
"""


import cfglib
import es
import gamethread
import langlib
import os.path
import playerlib
import random
import time

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'HealMates'
info.version  = '7'
info.basename = 'healmates'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'


###


config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/' + info.basename + '.cfg')

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename':info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')

config.text('***** Ability purchasing options *****')
cvar_ability_cost        = config.cvar('healmates_ability_cost',        2000,       'Cost of the ability to heal teammates--set to 0 for players to automatically receive the healing ability')
cvar_ability_buyzone     = config.cvar('healmates_ability_buyzone',     1,          '0 = players can buy the ability to heal teammates anywhere, 1 = players must be in a buy zone to purchase the ability to heal teammates')
cvar_ability_trigger     = config.cvar('healmates_ability_trigger',     '!buyheal', 'Command players must say to purchase the ability to heal teammates')
cvar_ability_loseondeath = config.cvar('healmates_ability_loseondeath', 1,          '0 = players keep the ability to heal players when killed, 1 = players lose the ability to heal players when killed')
config.text('\n')

config.text('***** General healing options *****')
cvar_heal_cost             = config.cvar('healmates_heal_cost',             10,  'Cost of healing another player one health point--set to 0 for no cost')
cvar_heal_maxgain          = config.cvar('healmates_heal_maxgain',          30,  'Maximum amount of health a player can gain from being healed')
cvar_heal_maxendhealth     = config.cvar('healmates_heal_maxendhealth',     85,  'Maximum amount of health a player can be healed to')
cvar_heal_distance         = config.cvar('healmates_heal_distance',         150, 'Maximum distance a player can be from a healing player to be healed')
cvar_heal_distance_percent = config.cvar('healmates_heal_distance_percent', 65,  'Percent of the health possible to gain from healing a player at the maximum distance will gain--set to 100 for players at the maximum healing distance to be healed the maximum amount')
cvar_heal_sound            = config.cvar('healmates_heal_sound',            35,  'Percent of the map that can hear when a player heals teammates--set to 0 for no sound')
cvar_heal_cheapbleed       = config.cvar('healmates_heal_cheapbleed',       1,   '0 = no change, 1 = when CheapBleed is enabled, healing a teammate also stops bleeding')
config.text('CheapBleed can be found here: http://addons.eventscripts.com/addons/view/cheapbleed')
config.text('\n')

config.text('***** Healing trigger options *****')
cvar_heal_trigger              = config.cvar('healmates_heal_trigger',              '!heal', 'Command players must say to heal teammates')
cvar_heal_trigger_attemptdelay = config.cvar('healmates_heal_trigger_attemptdelay', 4,       'Number of seconds required between heal attempts')
config.text('\n')

config.text('***** Automatic healing options *****')
cvar_heal_autoheal          = config.cvar('healmates_heal_autoheal',          0,    '0 = players must trigger healing of teammates, 1 = healing of teammates is handled automatically')
cvar_heal_autoheal_delay    = config.cvar('healmates_heal_autoheal_delay',    6,    'Number of seconds between forcing players with the healing ability to heal surrounding teammates--set to 0 to eliminate')
cvar_heal_autoheal_ecolimit = config.cvar('healmates_heal_autoheal_ecolimit', 6000, 'Minimum amount of money player will have after automatically healing')
config.text('Once players reach the minimum amount of money they will control their healing until they once again reach the minimum amount.')
config.text('\n')

config.text('***** Bot options *****')
cvar_bot               = config.cvar('healmates_bot',               1,    '0 = bots will neither buy the healing ability nor heal teammates randomly, 1 = bots will buy the healing ability and heal teammates randomly')
cvar_bot_buychance     = config.cvar('healmates_bot_buychance',     70,   'Percent chance that a bot with enough money will buy the ability to heal teammates')
cvar_bot_ecolimit_buy  = config.cvar('healmates_bot_ecolimit_buy',  8000, 'Amount of money a bot must have to attempt buying the healing ability')
cvar_bot_ecolimit_heal = config.cvar('healmates_bot_ecolimit_heal', 4000, 'Amount of money bots will refrain from spending on healing teammates')
cvar_bot_delay_min     = config.cvar('healmates_bot_delay_min',     4,    'Minimum number of seconds from healing teammates until a bot randomly heals teammates again')
cvar_bot_delay_max     = config.cvar('healmates_bot_delay_max',     25,   'Maximum number of seconds from healing teammates until a bot randomly heals teammates again')
config.text('\n')

config.text('***** Announce options *****')
cvar_announce_ability_command = config.cvar('healmates_announce_ability_command', 1, '0 = no change, 1 = announce command to purchase the healing ability each spawn')
config.cvar('healmates_announce_ability_chat',    0, '0 = no change, 1 = announce feedback from purchasing the healing ability in chat area')
config.cvar('healmates_announce_ability_center',  1, '0 = no change, 1 = announce feedback from purchasing the healing ability in center text')
config.cvar('healmates_announce_ability_hud',     1, '0 = no change, 1 = announce feedback from purchasing the healing ability in a HUD message')
config.text('')
cvar_announce_heal_command    = config.cvar('healmates_announce_heal_command',    2, '0 = no change, 1 = announce command to heal teammates only when a player acquires the healing ability, 2 = announce command to heal teammates each spawn')
config.cvar('healmates_announce_heal_chat',    0, '0 = no change, 1 = announce feedback from healing teammates in chat area')
config.cvar('healmates_announce_heal_center',  1, '0 = no change, 1 = announce feedback from healing teammates in center text')
config.cvar('healmates_announce_heal_hud',     1, '0 = no change, 1 = announce feedback from healing teammates in a HUD message')
cvar_announce_heal_individual = config.cvar('healmates_announce_heal_individual', 2, '0 = no change, 1 = announce to players each player they heal on the console, 2 = announce to players each player they heal in chat area')
config.text('')
cvar_announce_healed_chat   = config.cvar('healmates_announce_healed_chat',   2, '0 = no change, 1 = announce to players when they were healed by another player on the console, 2 = announce to players when they were healed by another player in chat area')
cvar_announce_healed_center = config.cvar('healmates_announce_healed_center', 0, '0 = no change, 1 = announce to players when they were healed by another player in center text')
cvar_announce_healed_hud    = config.cvar('healmates_announce_healed_hud',    1, '0 = no change, 1 = announce to players when they were healed by another player in a HUD message')
config.text('')
cvar_announce_heal_sameteam     = config.cvar('healmates_announce_heal_sameteam',     1, '0 = no change, 1 = announce to teammates when a player is healed by another player on the console, 2 = announce to teammates when a player is healed by another player in chat area')
cvar_announce_heal_opposingteam = config.cvar('healmates_announce_heal_opposingteam', 0, '0 = no change, 1 = announce to members of the opposing team when a player is healed by another player on the console, 2 = announce to members of the opposing team when a player is healed by another player in chat area')
config.text('\n')

config.text('***** Heal sound options *****')
config.text('')
config.text('Use the following server command to add heal sounds:')
config.text('')
config.text('healmates_addsound <"sound relative to ./sound/">')
config.text('')
config.text('Sounds are marked for download automatically.')
config.text('\n')

config.text('Place your healmates_addsound commands below:')
config.command('healmates_addsound')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('healmates_addsound "vo/npc/male01/health01.wav"')
config.text('healmates_addsound "vo/npc/male01/health02.wav"')
config.text('healmates_addsound "vo/npc/male01/health03.wav"')
config.text('healmates_addsound "vo/npc/male01/health04.wav"')
config.text('healmates_addsound "vo/npc/male01/health05.wav"')
config.text('\n')

config.text('Heal sounds can be removed with the following server command:')
config.text('')
config.text('healmates_removesound <"sound relative to ./sound/">')
config.text('')
config.text('Due to the fact heal sounds can be added and removed above, this command will not be used by most users')
config.text('\n')

config.text('***** Event documentation *****')
config.text('')
config.text('The following events will fire when HealMates is loaded:')
config.text('')
config.text('player_buyheal')
config.text('   event_var(userid)          - Player who bought the ability to heal teammates')
config.text('')
config.text('player_healed')
config.text('   event_var(userid)          - Player who was healed')
config.text('   event_var(attacker)        - Player who performed the healing')
config.text('   event_var(amount)          - Amount of health the healed player received')
config.text('')
config.text('These events also provide event_var(es_steamid), event_var(es_username), and all the other event_vars that accompany')
config.text('event_var(userid). More information can be found here: http://www.eventscripts.com/pages/Extended_event_variables')

config.write()


###


class HealSoundsManager(object):
   sounds    = []
   lastsound = ''
   soundpath = es.ServerVar('eventscripts_gamedir') + '/sound/'

   def __init__(self):
      self.sounds = []

   def addSound(self, sound):
      if sound not in self.sounds:
         self.sounds.append(sound)

   def removeSound(self, sound):
      if sound in self.sounds:
         self.sounds.remove(sound)

   def getSound(self):
      if self.sounds:
         choices        = filter(lambda x: x <> self.lastsound, self.sounds)
         self.lastsound = random.choice(choices if choices else self.sounds)

      else:
         self.lastsound = ''

      return self.lastsound

   def markSoundsForDownload(self):
      for sounds in filter(lambda x: os.path.isfile(self.soundpath + x), self.sounds):
         es.stringtable('downloadables', 'sound/' + sound)

healsounds = HealSoundsManager()


class HealMatesManager(object):
   userid      = 0
   is_bot      = False
   has_healing = False
   auto_heal   = False
   heal_limit  = 0

   def __init__(self, userid):
      self.userid = userid
      self.is_bot = es.getplayersteamid(userid) == 'BOT'

   def playerSpawn(self):
      ability_cost = int(cvar_ability_cost)
      player_lang  = playerlib.getPlayer(self.userid).get('lang')

      if not ability_cost and not self.has_healing:
         self.giveAbility(player_lang)

      elif self.has_healing:
         self.removeDelay()
         self._startAutoHeal()

         announce = int(cvar_announce_heal_command)
         if announce not in (0, 1):
            self.announceCommand(player_lang)

      elif self.is_bot and int(cvar_bot):
         bot_money = es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount')
         if bot_money >= cvar_bot_ecolimit_buy and bot_money >= ability_cost and random.randint(1, 100) <= cvar_bot_buychance:
            es.setplayerprop(self.userid, 'CCSPlayer.m_iAccount', bot_money - ability_cost)
            self.giveAbility(player_lang)

      elif int(cvar_announce_ability_command):
         es.tell(self.userid, '#multi', lang_text('ability purchase', {'purchase':say_commands['purchase'], 'cost':ability_cost}, player_lang))

   def playerDeath(self):
      self.removeDelay()
      if int(cvar_ability_loseondeath):
         self.has_healing = False

   def giveAbility(self, player_lang=''):
      if not player_lang: player_lang = playerlib.getPlayer(self.userid).get('lang')

      self.has_healing = True
      self.heal_limit  = 1

      es.event('initialize', 'player_buyheal')
      es.event('setint',     'player_buyheal', 'userid', self.userid)
      es.event('fire',       'player_buyheal')

      if int(cvar_announce_heal_command):
         self.announceCommand(player_lang)

      self._startAutoHeal()

   def _startAutoHeal(self):
      self.removeDelay()

      if int(cvar_heal_autoheal):
         self.auto_heal = True
         self._autoHeal()

      elif self.is_bot and int(cvar_bot):
         self.auto_heal = True
         self._botHeal()

   def _autoHeal(self):
      if 0 < self.heal_limit < time.time():
         money = es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount') - (min(int(cvar_heal_autoheal_ecolimit), int(cvar_bot_ecolimit_heal)) if self.is_bot else int(cvar_heal_autoheal_ecolimit))

         self.executeHeal(money)

      gamethread.delayedname(cvar_heal_autoheal_delay, 'healmates_%s' % self.userid, self._autoHeal)

   def _botHeal(self):  
      if 0 < self.heal_limit < time.time():
         money = es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount') - int(cvar_bot_ecolimit_heal)

         self.executeHeal(money)

      gamethread.delayedname(random.randint(int(cvar_bot_delay_min), int(cvar_bot_delay_max)), 'healmates_%s' % self.userid, self._botHeal)

   def buyCommand(self):
      player_lang = playerlib.getPlayer(self.userid).get('lang')

      if es.getplayerteam(self.userid) > 1 and not es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag'):
         if not self.has_healing:
            if es.getplayerprop(self.userid, 'CCSPlayer.m_bInBuyZone') % 2 or not int(cvar_ability_buyzone) or es.getentityindex('func_buyzone') == -1:

               ability_cost = int(cvar_ability_cost)
               money        = es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount')

               if money >= ability_cost:
                  es.setplayerprop(self.userid, 'CCSPlayer.m_iAccount', money - ability_cost)

                  self.giveAbility(player_lang)

                  self.commandFeedback(lang_text('ability success', {}, player_lang), 'ability')

               else:
                  self.commandFeedback(lang_text('ability cash', {'cash':ability_cost}, player_lang), 'ability')

            else:
               self.commandFeedback(lang_text('ability buy zone', {}, player_lang), 'ability')

         else:
            self.commandFeedback(lang_text('ability twice', {}, player_lang), 'ability')

      else:
         self.commandFeedback(lang_text('ability dead', {}, player_lang), 'ability')

   def healCommand(self, amount=0):
      player_lang = playerlib.getPlayer(self.userid).get('lang')

      if self.has_healing:
         if es.getplayerteam(self.userid) > 1 and not es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag'):
            money  = es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount')
            amount = money if amount > money or not amount else amount

            if 0 < self.heal_limit < time.time(): 
               if self.executeHeal(amount) == -1:
                  self.commandFeedback(lang_text('heal cash', {'cash':int(cvar_heal_cost)}, player_lang), 'heal')

            else:
               self.commandFeedback(lang_text('heal limit', {}, player_lang), 'heal')

         else:
            self.commandFeedback(lang_text('heal dead', {}, player_lang), 'heal')

      else:
         self.commandFeedback(lang_text('heal fail', {}, player_lang), 'heal')

   def executeHeal(self, money, player_lang=''):
      """
      Returns -1 if not enough money for one health unit, otherwise returns amount of heal given
      """
      unit_cost = int(cvar_heal_cost)
      if money < unit_cost:
         return -1

      near_players = self._getNearPlayers()
      if not near_players: return 0

      health_given = 0
      amount_spent = 0
      end_health   = int(cvar_heal_maxendhealth)

      near_possible_players = set(near_players)
      while (amount_spent + unit_cost <= money or not unit_cost) and near_possible_players:
         lowest_health = min(map(lambda x: near_players[x]['health'], near_possible_players))
         if lowest_health >= end_health: break

         lowest_userid  = filter(lambda x: near_players[x]['health'] == lowest_health, near_possible_players)[0]
         current_player = near_players[lowest_userid]

         current_player['possible'] -= 1
         current_player['given']    += 1
         current_player['health']   += 1

         es.setplayerprop(lowest_userid, 'CBasePlayer.m_iHealth', lowest_health + 1)

         amount_spent += unit_cost
         health_given += 1

         if not current_player['possible']:
            near_possible_players.remove(lowest_userid)

      if not health_given: return 0

      self.heal_limit = time.time() + float(cvar_heal_trigger_attemptdelay)

      if amount_spent:
         es.setplayerprop(self.userid, 'CCSPlayer.m_iAccount', es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount') - amount_spent)

      announce_healer        = int(cvar_announce_heal_individual)
      announce_healed_chat   = int(cvar_announce_healed_chat)
      announce_healed_center = int(cvar_announce_healed_center)
      announce_healed_hud    = int(cvar_announce_healed_hud)
      announce_sameteam      = int(cvar_announce_heal_sameteam)
      announce_opposingteam  = int(cvar_announce_heal_opposingteam)

      if not player_lang: player_lang = playerlib.getPlayer(self.userid).get('lang')
      healer_text = 'healer cost' if unit_cost else 'healer no cost'
      tokens      = {'player':'', 'health':0, 'healer':es.getplayername(self.userid), 'cost':0}
      team        = es.getplayerteam(self.userid)
      stop_bleed  = self._getCheapBleed()

      for userid in near_players:
         health = tokens['health'] = near_players[userid]['given']
         if health:
            tokens['player'] = es.getplayername(userid)
            tokens['cost']   = health * unit_cost

            es.event('initialize', 'player_healed')
            es.event('setint',     'player_healed', 'userid',   userid)
            es.event('setint',     'player_healed', 'attacker', self.userid)
            es.event('setint',     'player_healed', 'amount',   health)
            es.event('fire',       'player_healed')

            stop_bleed(userid)

            loop_lang = playerlib.getPlayer(userid).get('lang')

            if announce_healer == 1:
               es.cexec(self.userid, 'echo ' + self.__removeTags(lang_text(healer_text, tokens, player_lang)))
            elif announce_healer:
               es.tell(self.userid, '#multi', lang_text(healer_text, tokens, player_lang))

            if announce_healed_chat == 1:
               es.cexec(userid, 'echo ' + self.__removeTags(lang_text('healed', tokens, loop_lang)))
            elif announce_healed_chat:
               es.tell(userid, '#multi', lang_text('healed', tokens, loop_lang))

            if announce_healed_center:
               es.centertell(userid, self.__removeTags(lang_text('healed', tokens, loop_lang)))

            if announce_healed_hud:
               getPlayer(userid)._hudHint(self.__removeTags(lang_text('healed', tokens, loop_lang)))

            if announce_opposingteam or announce_sameteam:
               for loop_userid in es.getUseridList():
                  loop_team = es.getplayerteam(loop_userid)
                  if team == 5 - loop_team:
                     if announce_opposingteam == 1:
                        es.cexec(loop_userid, 'echo ' + self.__removeTags(lang_text('heal broadcast', tokens, playerlib.getPlayer(loop_userid).get('lang'))))
                     elif announce_opposingteam:
                        es.tell(loop_userid, '#multi', lang_text('heal broadcast', tokens, playerlib.getPlayer(loop_userid).get('lang')))

                  elif loop_userid not in (self.userid, userid):
                     if announce_sameteam == 1:
                        es.cexec(loop_userid, 'echo ' + self.__removeTags(lang_text('heal broadcast', tokens, playerlib.getPlayer(loop_userid).get('lang'))))
                     elif announce_sameteam:
                        es.tell(loop_userid, '#multi', lang_text('heal broadcast', tokens, playerlib.getPlayer(loop_userid).get('lang')))

      self._emitSound()

      return health_given

   def _getNearPlayers(self):
      team         = es.getplayerteam(self.userid)
      location     = es.getplayerlocation(self.userid)
      near_players = {}

      heal_distance = float(cvar_heal_distance) + 40
      start_percent = int(cvar_heal_distance_percent)
      max_gain      = int(cvar_heal_maxgain)

      for userid in filter(lambda x: self.userid <> x and es.getplayerteam(x) == team and not es.getplayerprop(x, 'CBasePlayer.pl.deadflag'), es.getUseridList()):
         loop_location = es.getplayerlocation(userid)
         loop_distance = sum(((location[x] - loop_location[x]) ** 2 for x in range(3))) ** 0.5
         if loop_distance <= heal_distance:
            loop_max_gain = int(float((100. - start_percent) * (heal_distance - loop_distance) / heal_distance + start_percent) * max_gain / 100.)

            if not loop_max_gain and is_bleeding(userid): loop_max_gain = 1
            if loop_max_gain:
               near_players[userid] = {'possible':loop_max_gain, 'given':0, 'health':es.getplayerprop(userid, 'CBasePlayer.m_iHealth')}

      return near_players

   def _emitSound(self):
      float_sound = float(cvar_heal_sound)
      if not float_sound: return

      sound = healsounds.getSound()
      if not sound: return

      es.emitsound('player', self.userid, sound, 1.0, (100 - float_sound) / 100)

   def _getCheapBleed(self):
      if int(cvar_heal_cheapbleed):
         try:
            cheapbleed = es.import_addon('cheapbleed')
            return lambda x: cheapbleed.removePlayer(x)

         except KeyError: pass

      return lambda x: x

   def removeDelay(self):
      gamethread.cancelDelayed('healmates_%s' % self.userid)
      if self.auto_heal:
         self.auto_heal = False

   def announceCommand(self, player_lang):
      es.tell(self.userid, '#multi', lang_text('heal teammates', say_commands, player_lang))
      heal_cost = int(cvar_heal_cost)
      if heal_cost:
         es.tell(self.userid, '#multi', lang_text('heal cost', {'cost':heal_cost}, player_lang))

   def commandFeedback(self, message, command_type):
      """Displays the message to the player in the specified langauge"""
      if int(es.ServerVar('healmates_announce_%s_chat' % command_type)):
         es.tell(self.userid, '#multi', message)

      if int(es.ServerVar('healmates_announce_%s_center' % command_type)):
         es.centertell(self.userid, self.__removeTags(message))

      if int(es.ServerVar('healmates_announce_%s_hud' % command_type)):
         self._hudHint(self.__removeTags(message))

   def _hudHint(self, message):
      """Sends the player a HUD hint"""
      es.usermsg('create', 'healmates_hud', 'HintText')
      es.usermsg('write', 'short', 'healmates_hud', -1)
      es.usermsg('write', 'string', 'healmates_hud', message)
      es.usermsg('send', 'healmates_hud', self.userid)
      es.usermsg('delete', 'healmates_hud')

   def __removeTags(self, text):
      """Removes #lightgreen, #green and #default tags from the supplied string"""
      return text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


players = {}


def getPlayer(userid):
   userid = int(userid)
   if userid not in players:
      players[userid] = HealMatesManager(userid)

   return players[userid]


def removePlayer(userid):
   userid = int(userid)
   if userid in players:
      players[userid].removeDelay()
      del players[userid]


def clearPlayers():
   for userid in players:
      players[userid].removeDelay()
   players.clear()


###


def getLangText():
   lang_path = es.getAddonPath(info.basename) + '/%s_languages.ini' % info.basename
   if os.path.isfile(lang_path):
      return langlib.Strings(lang_path)

   else:
      es.dbgmsg(0, '%(name)s: Unable to load %(basename)s_languages.ini! Please ensure it is in the ./%(basename)s/ directory.' % {'name':info.name, 'basename':info.basename})
      return lambda x, y, z: '%s: %s_languages.ini not found' % (info.name, info.basename)

lang_text = getLangText()


###


say_commands = {'purchase':'', 'heal':''}


def load():
   """
   Checks to make sure critical server variables are created by the config
   Calls load_events
   Creates say commands for triggering healing
   """
   config.execute()

   command = str(cvar_ability_trigger).lower()
   if command:
      if not es.exists('saycommand', command):
         es.regsaycmd(command, 'healmates/buy_cmd', 'Say command for buying the ability to heal teammates')
         say_commands['purchase'] = command

   command = str(cvar_heal_trigger).lower()
   if command:
      if not es.exists('saycommand', command):
         es.regsaycmd(command, 'healmates/heal_cmd', 'Say command for healing surrounding teammates')
         say_commands['heal'] = command

   healsounds.markSoundsForDownload()

   load_events()


def es_map_start(event_var):
   clearPlayers()

   healsounds.markSoundsForDownload()

   load_events()


def player_spawn(event_var):
   """
   Displays the message telling players how to buy or use the healing ability
   Handles bots buying the healing ability
   """
   if int(event_var['es_userteam']) < 2: return

   getPlayer(event_var['userid']).playerSpawn()


def player_death(event_var):
   """Removes delays for the dead player"""
   getPlayer(event_var['userid']).playerDeath()


def player_disconnect(event_var):
   """Removes delays and dictionary element for the dead player"""
   removePlayer(event_var['userid'])


def unload():
   clearPlayers()

   for command in filter(lambda x: bool(say_commands[x]), say_commands):
      es.unregsaycmd(say_commands[command])

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)


###


def buy_cmd():
   """Calls give_ability for the player buying the healing ability"""
   getPlayer(es.getcmduserid()).buyCommand()


def heal_cmd():
   """
   Triggers execute_heal for the player with the desired arguments
   Starts a delay on the player that calls remove_restriction so the player cannot use this command until the delay expires
   """
   amount = 0
   if es.getargc() > 1:
      arg    = es.getargv(1)
      amount = int(arg) if arg.isdigit() else 0

   getPlayer(es.getcmduserid()).healCommand(amount)


###


def addsound_cmd():
   """
   healmates_addsound <"sound relative to ./sound/">
   """
   if es.getargc() == 2:
      healsounds.addSound(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: healmates_addsound <\"sound relative to ./sound/\">')

if not es.exists('command', 'healmates_addsound'):
   es.regcmd('healmates_addsound', 'healmates/addsound_cmd', 'healmates_addsound <\"sound relative to ./sound/\">\nCommand to add heal sounds')


def removesound_cmd():
   """
   healmates_removesound <"sound relative to ./sound/">
   """
   if es.getargc() == 2:
      healsounds.removeSound(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: healmates_removesound <\"sound relative to ./sound/\">')

if not es.exists('command', 'healmates_removesound'):
   es.regcmd('healmates_removesound', 'healmates/removesound_cmd', 'healmates_removesound <\"sound relative to ./sound/\">\nCommand to remove heal sounds')


###


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('healmates') + '/healmates.res'):
      es.loadevents('declare', 'addons/eventscripts/healmates/healmates.res')

   else:
      events = {'player_buyheal':{'userid':'short'}, 'player_healed':{'userid':'short', 'attacker':'short', 'amount':'short'}}

      if not os.path.isfile(es.getAddonPath('healmates') + 'es_healmates_events_db.txt'):
         es.keygroupcreate('healmates_events')
         for event_name in events:
            es.keycreate('healmates_events', event_name)
            for event_var in events[event_name]:
               es.keysetvalue('healmates_events', event_name, event_var, events[event_name][event_var])
         es.keygroupsave('healmates_events', '|healmates')
         es.keygroupdelete('healmates_events')
      es.loadevents('declare', 'addons/eventscripts/healmates/es_healmates_events_db.txt')
      es.dbgmsg(0, 'HealMates: Unable to load healmates.res! Please ensure it is in the ./healmates/ directory.')